const { timingSafeEqual } = require('crypto');
const { readFileSync, readdirSync } = require('fs');
const { inspect } = require('util');
const { exec } = require('child_process');

const ssh2 = require('ssh2');

const allowedUser = Buffer.from(process.env.SSH_USER || 'user');
const allowedPassword = Buffer.from(process.env.SSH_PASSWORD || 'docker-ssh-password');
const allowedPubKey = ssh2.utils.parseKey(process.env.SSH_PUB_KEY_FILE || readFileSync('keys/docker-ssh.pub'), process.env.SSH_PUB_KEY_PASSPHRASE);

function checkValue(input, allowed) {
  const autoReject = (input.length !== allowed.length);
  if (autoReject) {
    // Prevent leaking length information by always making a comparison with the
    // same input when lengths don't match what we expect ...
    allowed = input;
  }
  const isMatch = timingSafeEqual(input, allowed);
  return (!autoReject && isMatch);
}

new ssh2.Server({
  hostKeys: [readFileSync('keys/docker-ssh')]
}, (client) => {
  console.log('Client connected!');

  client.on('authentication', (ctx) => {
    let allowed = true;
    if (!checkValue(Buffer.from(ctx.username), allowedUser))
      allowed = false;

    switch (ctx.method) {
      case 'password':
        if (!checkValue(Buffer.from(ctx.password), allowedPassword))
          return ctx.reject();
        break;
      case 'publickey':
        if (ctx.key.algo !== allowedPubKey.type
            || !checkValue(ctx.key.data, allowedPubKey.getPublicSSH())
            || (ctx.signature && allowedPubKey.verify(ctx.blob, ctx.signature) !== true)) {
          return ctx.reject();
        }
        break;
      default:
        return ctx.reject();
    }

    if (allowed)
      ctx.accept();
    else
      ctx.reject();
  }).on('ready', () => {
    console.log('Client authenticated!');

    client.on('session', (accept, reject) => {
      const session = accept();
      session.on('pty', (accept, reject, info) => {
        accept();
      });
      
      session.once('exec', (accept, reject, info) => {
        console.log('Client wants to execute: ' + inspect(info.command));
        
        try {
          const stream = accept();
          const child = exec(info.command);          
          let finished = false;
          child.stdout.on('data', (data) => {
            console.log(data);
            stream.stdin.write(data);
          });

          child.stderr.on('data', (data) => {
            stream.stderr.write(data);
          });
          child.on('error', (error) => {
            stream.stderr.write(error.toString());
          });

          child.once('close', (code) => {
            finished = true;
            setTimeout(() => {
              stream.exit(code);
              stream.end();
            }, 500);
          });

          stream.stdout.on('data', (data) => {
            console.log(data.toString());
            if (!finished) {
              child.stdin.write(data.toString());
            }
          });
        } catch (err) {
          console.error(err);
        }        
      });
      
    });
  }).on('close', () => {
    console.log('Client disconnected');
  });
}).listen(process.env.PORT || 2222, process.env.HOST || '127.0.0.1', function() {
  console.log('Listening on port ' + this.address().port);
});