const databaseDup = require('./steps/duplicate-database');
const fileDup = require('./steps/duplicate-file');
const { selectRGDS } = require('./rgds');
const { readFileSync, writeFileSync } = require('fs');
const inquirer = require('inquirer');
const { spawnSync } = require('child_process');
const chalk = require('chalk');

const { logger } = require('./logger');


exports.run = async (projects, options) => {
  if (!options.mode) options.mode = 'all';
  let valid = false;
  try {
    if (!projects || !projects.length) projects = await selectRGDS();
    console.log(chalk.blue('Start stepper run with inputs'));
    console.log(chalk.blue('Projects'), projects);

    let parralelPromises = [];
    if (projects.length) {
      for (const project of projects) {
        if (options.parralel) {
          parralelPromises.push(runSingleRGDS(project, logger.configure(project), options));
        } else {
          await runSingleRGDS(project, logger.configure(project), options);
          console.log(chalk.bold.cyan('-'.repeat(80)));
        }
      }

      if (options.parralel) await Promise.all(parralelPromises);
    } else {
      throw new Error('no projects selected');
    }
    
  } catch(err) {
    console.error(chalk.red(`Error during duplications process`));
    console.error(err);
    valid = false;
  }  
  console.log(chalk.blue('End stepper run'));

  return valid;
};

const runSingleRGDS = async (project, currentLogger, options) => {
  let valid = false;
  try {
    currentLogger.info('Project', project);
    console.log(chalk.blue('Project'), project);
    currentLogger.info('Verifications');
    console.log(chalk.blue('Verifications'));

    if (options.only !== 'db') {
      valid = await fileDup.verify(project, currentLogger, options.ssh);
      if (!valid) {
        throw new Error('File duplication verification failed');
      }
    }

    if (options.only !== 'file') {
      valid = await databaseDup.verify(project, currentLogger);
      if (!valid) {
        throw new Error('Database duplication verification failed');
      }
    }

    currentLogger.info('Duplications');
    console.log(chalk.blue('Duplications'));

    if (options.only !== 'db') {
      valid = await fileDup.run(project, currentLogger, options.ssh);
      if (!valid) {
        throw new Error('File duplication failed');
      }
    }

    if (options.only !== 'file') {
      valid = await databaseDup.run(project, currentLogger);
      if (!valid) {
        throw new Error('Database duplication failed');
      } 
    }

    currentLogger.info('Run process executed succesfully');
    console.log(chalk.green('Run process executed succesfully'));
    
  } catch(err) {
    currentLogger.error(`Error during duplication process `);
    console.error(chalk.red(`Error during duplication process `));
    currentLogger.error(err);
    console.error(err);
    valid = false;
  }
  return valid;
}

const questions = {
  ".source.database.host" : "Enter host from Mysql Source server",
  ".source.database.port" : "Enter port from Mysql Source server",
  ".source.database.user" : "Enter user from Mysql Source server",
  ".source.database.password" : "Enter password from Mysql Source server",
  ".source.database.defaultDb" : "Enter default database name from Mysql Source server",
  ".source.database.ssl" : "Enter value ( true or false ) to activate SSL communication with Mysql Source server",
  ".target.database.host": "Enter host from Mysql Target server",
  ".target.database.port": "Enter port from Mysql Target server",
  ".target.database.user" : "Enter user from Mysql Target server",
  ".target.database.password" : "Enter password from Mysql Target server",
  ".target.database.defaultDb" : "Enter default database name from Mysql Target server",
  ".target.database.ssl" : "Enter value ( true or false ) to activate SSL communication with Mysql Target server",
  ".source.file.azure.url": "Enter Azure Source storage HTTP URL ( leave blank if using local filsystem )",
  ".source.file.azure.token": "Enter Azure Source storage SAS token ( leave blank if using local filsystem )",
  ".source.file.basePath": "Enter Source absolute path to RGDS's files ( leave blank if using Azure storage )",
  ".target.file.azure.url": "Enter Azure Source storage HTTP URL ( leave blank if using local filsystem )",
  ".target.file.azure.token": "Enter Azure Source storage SAS token ( leave blank if using local filsystem )",
  ".target.file.basePath": "Enter Source absolute path to RGDS's files ( leave blank if using Azure storage )",
}

const askValue = async (title, value) => {
  const answer = await inquirer.prompt([
    {
      type: 'input',
      name: 'Configuration',
      message: questions[title],
      default: value,
    }
  ]);
  return answer['Configuration'];
}

const configureKey = async (conf, keyPath = "") => {
  for (const key in conf) {
    if (typeof conf[key] === 'object') {
      await configureKey(conf[key], `${keyPath}.${key}`);
    } else {
      conf[key] = await askValue(`${keyPath}.${key}`, conf[key]);
    }
  }
}

exports.configure = async () => {
  const conf = JSON.parse(readFileSync(__dirname + "/../configuration.json"));
  await configureKey(conf);
  writeFileSync(__dirname + "/../configuration.json", JSON.stringify(conf, null, 4));
}

exports.install = () => {
  const check = spawnSync('azcopy', ['-h']);
  console.log(chalk.yellow('AZCopy binaries checked'));
  if (check.error) {
    console.error(chalk.red('AZCopy binaries missing'));
    spawnSync('curl', ['-L', '-o', '/tmp/azcopy.tar.gz', 'https://aka.ms/downloadazcopy-v10-linux']);
    spawnSync('tar', ['-xf', '/tmp/azcopy.tar.gz', '--strip-components=1']);
    spawnSync('cp',  ['./azcopy', '/usr/local/bin/azcopy']);
    spawnSync('chmod', ['775', '/usr/local/bin/azcopy']);
    spawnSync('rm', ['/tmp/azcopy.tar.gz']);
    spawnSync('rm', ['./azcopy']);
    const check = spawnSync('azcopy', ['-h']);
    if (!check.error) {
      console.log(chalk.green('AZCopy binaries checked successfully'));
    } else {
      console.error(chalk.red('AZCopy binaries still missing after install attempt'));
    }
  } else {
    console.log(chalk.green('AZCopy binaries checked successfully'));
  }

  const checkMysql = spawnSync('mysqldump', ['--help']);
  console.log(chalk.yellow('MysqlDump binaries checked'));
  if (checkMysql.error) {
    console.error(chalk.red('MysqlDump binaries missing'));
    console.error(chalk.yellow('Please install package mysql-client'));
    
  } else {
    console.log(chalk.green('MysqlDump binaries checked successfully'));
  }
}